--[[ *** LOAD LIBRARIES *** ]]--
local Guilder = _G.Guilder
if not Guilder then
	error("Guilder_BusyResponder requires Guilder")
end

local Guilder_BusyResponder = Guilder:NewModule("BusyResponder")
local AL = Guilder.AL
Guilder_BusyResponder.modName = AL["Busy Responder"]

--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		message = AL["RESPONDER_MSG"],
		officer = true,
		guild = true,
		party = true,
		raid = true,
		friend = true,
	},
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	RespondToOfficers = {
		type = "toggle",
		name = AL["Respond to Guild Officers"],
		desc = AL["Choose whether the responder should respond to officers of your guild"],
		width = "full",
		order = 20,
		get = function() return Guilder_BusyResponder.db.profile.officer end,
		set = function(info, v) Guilder_BusyResponder.db.profile.officer = v end,
	},
	RespondToMembers = {
		type = "toggle",
		name = AL["Respond to Guild Members"],
		desc = AL["Choose whether the responder should respond to members of your guild"],
		width = "full",
		order = 21,
		get = function() return Guilder_BusyResponder.db.profile.guild end,
		set = function(info, v) Guilder_BusyResponder.db.profile.guild = v end,
	},
	RespondToParty = {
		type = "toggle",
		name = AL["Respond to Party Members"],
		desc = AL["Choose whether the responder should respond to members of your party"],
		width = "full",
		order = 22,
		get = function() return Guilder_BusyResponder.db.profile.party end,
		set = function(info, v) Guilder_BusyResponder.db.profile.party = v end,
	},
	RespondToRaid = {
		type = "toggle",
		name = AL["Respond to Raid Members"],
		desc = AL["Choose whether the responder should respond to members of your raid"],
		width = "full",
		order = 23,
		get = function() return Guilder_BusyResponder.db.profile.raid end,
		set = function(info, v) Guilder_BusyResponder.db.profile.raid = v end,
	},
	RespondToFriends = {
		type = "toggle",
		name = AL["Respond to Friends"],
		desc = AL["Choose whether the responder should respond to members of your friends-list"],
		width = "full",
		order = 24,
		get = function() return Guilder_BusyResponder.db.profile.friend end,
		set = function(info, v) Guilder_BusyResponder.db.profile.friend = v end,
	},
	CustomiseMessage = {
		type = "input",
		name = AL["Customise Message"],
		desc = AL["Customise the message sent by the responder"],
		width = "full",
		order = 25,
		get = function() return Guilder_BusyResponder.db.profile.message end,
		set = function(info, v) Guilder_BusyResponder.db.profile.message = v end,
	},
	ResetMessage = {
		type = "execute",
		name = AL["Reset Message"],
		desc = AL["Reset the message to the default message originally supplied"],
		order = 26,
		func = function() Guilder_BusyResponder.db.profile.message = AL["RESPONDER_MSG"] end,
	},
	TestMessage = {
		type = "execute",
		name = AL["Test Message"],
		desc = AL["Test the BusyResponder message by whispering it to yourself"],
		order = 27,
		func = function()
			Guilder:SendMessage("BusyResponder - "..Guilder_BusyResponder.db.profile.message, "WHISPER", UnitName("player"))
		end,
	},
}

--[[ *** MODULE FUNCTIONS *** ]]--

function Guilder_BusyResponder:OnInitialize()
	local db = Guilder.db:RegisterNamespace("Busy Responder", defaults)
	self.db = db
end

function Guilder_BusyResponder:OnEnable()
end

function Guilder_BusyResponder:OnDisable()
end

function Guilder_BusyResponder:Info()
	return AL["Auto-respond to whispers with a busy message with this tool"]
end

function Guilder_BusyResponder:GetOptions()
	return options
end

--[[ *** BUSY RESPONDER FUNCTIONS *** ]]--

function Guilder_BusyResponder:SendBusyResponder(sender, msg)
--[[
	SendBusyResponder(sender, msg)

	This function aims to send a response to the 'sender' of the whispered message that was
	received. Before it sends the response however, it first checks whether the sender is in one of
	several possible groups that are currently excluded from receiving Responder messages.

	Current flags the responder can be configured to notice:

	If the sender is in our guild.
	If the sender is an officer of our guild.
	If the sender is in our party.
	If the sender is in our raid.
	If the sender is on our friends-list.

	Each of these options may be toggled by the user individually.
]]
	-- Perform the guild-member exclusion check.
	if LG:HasMember(sender) and Guilder_BusyResponder.db.profile.guild == false then
		return
	end
	-- Perform the officer-member exclusion check.
	if LG:HasMember(sender) and Guilder:IsOfficer(sender) and Guilder_BusyResponder.db.profile.officer == false then
		return
	end
	-- Perform the party-member exclusion check.
	if UnitInParty(sender) and Guilder_BusyResponder.db.profile.party == false then
		return
	end
	-- Perform the raid-member exclusion check.
	if UnitInRaid(sender) and Guilder_BusyResponder.db.profile.raid == false then
		return
	end
	-- Perform the friends-list exclusion check.
	if Guilder:UnitInFriends(sender) and Guilder_BusyResponder.db.profile.friend == false then
		return
	end
	-- Perform the "responding to a [BusyResponder] message" check.
	local brMessageTest = string.find(msg:lower(), "^%[guilder%] busyresponder")
	if brMessageTest then
		return
	end
	-- Send the response message.
	Guilder:SendMessage("BusyResponder - "..Guilder_BusyResponder.db.profile.message, "WHISPER", sender)
end

