--[[ *** LOAD LIBRARIES *** ]]--
local Guilder = _G.Guilder
if not Guilder then
	error("Guilder_VersionCheck requires Guilder")
end

local Guilder_VersionCheck = Guilder:NewModule("VersionCheck", "AceComm-3.0")
local AL = Guilder.AL
Guilder_VersionCheck.modName = AL["Version Check"]


--[[ *** SET UP VARIABLES *** ]]--


--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		outputTo = "Notice",
		updateAlert = true,
	}
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	display_results = {
		type = "select",
		name = AL["Display in"],
		desc = AL["Set whether version check results are shown in the chat window or a popup window"],
		values = { ["Notice"] = AL["Chat Window"], ["Popup"] = AL["Popup Window"], },
		order = 21,
		get = function() return Guilder_VersionCheck.db.profile.outputTo end,
		set = function(info, v) Guilder_VersionCheck.db.profile.outputTo = v end,
	},
	display_alert = {
		type = "toggle",
		name = AL["Alert on update"],
		desc = AL["Set whether or not to alert you if there is a newer version available"],
		order = 22,
		get = function() return Guilder_VersionCheck.db.profile.updateAlert end,
		set = function(info, v) Guilder_VersionCheck.db.profile.updateAlert = v end,
	},
	run = {
		type = "execute",
		name = AL["Perform a version check"],
		desc = AL["Performs the version check"],
		order = 23,
		width = "full",
		func = function() Guilder_VersionCheck:DoVersionCheck() end,
	},
}

--[[ *** MODULE FUNCTIONS *** ]]--

function Guilder_VersionCheck:OnInitialize()
	local db = Guilder.db:RegisterNamespace("Version Check", defaults)
	self.db = db
end

function Guilder_VersionCheck:OnEnable()
	self:RegisterComm("GLDRVCQ", "VersionCheckQuery")
	self:RegisterComm("GLDRVCR", "VersionCheckResponse")
end

function Guilder_VersionCheck:OnDisable()
	self:UnregisterComm("GLDRVCQ")
	self:UnregisterComm("GLDRVCR")
end

function Guilder_VersionCheck:Info()
	return AL["Compare your version of the addon to that of others in your raid, party or guild"]
end

function Guilder_VersionCheck:GetOptions()
	return options
end

--[[ *** CATCH EVENTS *** ]]--

--[[ *** VERSION CHECKER FUNCTIONS *** ]]--

function Guilder_VersionCheck:VersionCheckQuery(prefix, message, distribution, sender)
--[[
	This function processes the inter-mod version check request event. It constructs a response
	containing the version number of the instance of this mod being run by the user and sends the
	response out via the mod distribution, using the same distribution-focus as the receive message.

	Message format: RECIPIENT-VERSIONNUMBER
	RECIPIENT:      The person to whom this response is targeted.
]]
	if sender == myName then
		return
	end
	self:SendCommMessage("GLDRVCR", myName.."-"..myVersionNumber, "WHISPER", sender)
	-- Notify user if there is a newer version available
	if Guilder.db.profile.modules["Version Check"] and db.profile.updateAlert then
		-- Decode the message text into its component parts.
		local Args = Guilder:GetArgs(message, "-")
		local recipient = Args[1]
		local versionNumber = Args[2]
		if (versionNumber > myVersionNumber) then
			if db.profile.outputTo == "Notice" then
				Guilder:Print(BLIZZARD..AL["Newer version detected!"])
				Guilder:Print(WHITE..myName..": "..BLIZZARD..myVersionNumber)
				Guilder:Print(WHITE..sender..": "..GREEN..versionNumber)
			else
				msgFrame:PrepareForVersionCheck()
				msgFrame:AddLine(sender, GREEN..versionNumber)
				msgFrame:Show()
			end
		end
	end
end

function Guilder_VersionCheck:VersionCheckResponse(prefix, message, distribution, sender)
--[[
	This function processes the inter-mod version check response event. It determines whether
	this is a response sent to the user and if it is it then displays that response in the user's
	chat window.
]]
	-- Confirm whether we are expected to respond to this message.
	if recipient ~= nyName then
		return
	end

	-- Decode the message text into its component parts.
	local Args = Guilder:GetArgs(message, "-")
	local recipient = Args[1]
	local versionNumber = Args[2]

	-- Pre-colour the version number, w.r.t. this version.
	local versionText = YELLOW..versionNumber
	if versionNumber > myVersionNumber then
		versionText = GREEN..versionNumber
	end
	if versionNumber < myVersionNumber then
		versionText = RED..versionNumber
	end

	-- Perform the response for a standard version-check.
	if db.profile.outputTo == "Notice" then
		Guilder:Print(WHITE..sender..": "..versionText)
	else
		msgFrame:AddLine(sender, versionText)
		msgFrame:Show()
	end
end

function Guilder_VersionCheck:DoVersionCheck()
--[[
	This function outputs an inter-mod message to trigger a version check response from any other
	instances of mod that are within 'earshot', depending upon the distribution on which the message
	is sent out on.
]]
	if Guilder.db.profile.modules["Version Check"] then
		local dist
		if GetNumRaidMembers() > 0 then
			dist = "RAID"
		elseif GetNumPartyMembers() > 0 then
			dist = "PARTY"
		elseif IsInGuild() then
			dist = "GUILD"
		else
			return
		end
		Guilder:Print(AL["Performing a Guilder version check"].." ("..string.lower(dist)..")")
		if db.profile.outputTo == "Notice" then
			Guilder:Print(WHITE..myName..": "..BLIZZARD..myVersionNumber)
		else
			msgFrame:PrepareForVersionCheck()
		end
		self:SendCommMessage("GLDRVCQ", myName.."-"..myVersionNumber, dist)
	end
end
